/*
 * UART_Application1.c
 *
 * Created: 2016/10/24 下午 04:45:53
 * Author : Calvin Ho
 */ 

#define FOSC 16000000 // Clock Speed
#define BAUD 9600
#define MYUBRR FOSC/16/BAUD-1

#include <avr/io.h>
#include <avr/interrupt.h>

#define LED_ON	PORTB |= (1<<PORTB5)
#define LED_OFF PORTB &= ~(1<<PORTB5)
#define LED_Tog	PINB  |= (1<<PINB5)

#define LED11_ON	PORTD |= (1<<PORTD4) 
#define LED11_OFF	PORTD &= ~(1<<PORTD4)
#define LED12_ON	PORTD |= (1<<PORTD5) 
#define LED12_OFF	PORTD &= ~(1<<PORTD5)
#define LED13_ON	PORTD |= (1<<PORTD6) 
#define LED13_OFF	PORTD &= ~(1<<PORTD6)
#define LED14_ON	PORTD |= (1<<PORTD7) 
#define LED14_OFF	PORTD &= ~(1<<PORTD7)

#define SW4_PRESSED		!(PINB & (1<<PINB7))
#define SW5_PRESSED		!(PIND & (1<<PIND3))

unsigned int	ADC_Value ;
unsigned int	TIMER0_Counter = 0 ;
unsigned int	UART_Counter = 0 ;
unsigned char	ADC_LowByte, ADC_HighByte ;
unsigned char	PINB_Value ;
unsigned char	PIND_Value ;
unsigned char	My_Buffer[20] ;


ISR(ADC_vect)
{

	
	ADC_LowByte = ADCL ;
	ADC_HighByte = ADCH ;
			
	ADC_Value = ADC_HighByte * 256 + ADC_LowByte ;

}

ISR(TIMER0_COMPA_vect)
{
	
}

ISR(TIMER0_OVF_vect)
{
	TIMER0_Counter ++ ;
	UART_Counter ++ ;
	if (TIMER0_Counter >= (ADC_Value/8) )
		{
			LED_Tog ;
			TIMER0_Counter = 0 ;
		}
}

void	TIMER0_Init(void)
{
		TCCR0B |= (1<<CS01) | (1<<CS00) ;
		TIMSK0 |= (1<<TOIE0) ;
		
		TCNT0 = 0 ;
}
void	ADC_Init(void)
{
	ADMUX |= (1<<REFS0) | (1<<MUX1) | (1<<MUX2) ;
	ADCSRA |= (1<<ADEN) |(1<<ADSC) | (1<<ADATE) | (1<<ADIE) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0) ;
}

unsigned char USART0_Receive( void )
{
	/* Wait for data to be received */
	while ( !(UCSR0A & (1<<RXC0)) )
	;
	/* Get and return received data from buffer */
	return UDR0 ;
}

void USART0_Transmit( unsigned char data )
{
	/* Wait for empty transmit buffer */
	while ( !( UCSR0A & (1<<UDRE0)) )
	;
	/* Put data into buffer, sends the data */
	UDR0 = data;
}

void USART0_Init( unsigned int ubrr)
{
	/*Set baud rate */
	UBRR0H = (unsigned char)(ubrr>>8);
	UBRR0L = (unsigned char)ubrr;
	/*Enable receiver and transmitter */
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	/* Set frame format: 8data, 1stop bit */
	UCSR0C = (3<<UCSZ00);
	}

void	USART0_PutString(unsigned char* TheStringPtr)
{
		while (*TheStringPtr != 0x00 )
		{
			USART0_Transmit((*TheStringPtr));
			TheStringPtr ++ ;
		}
	
}

int main(void)
{
	DDRB |= (1<<DDRB5) ;
	DDRB &= ~(1<<DDRB7) ;
	
	DDRD |= (1<<DDRD4) ;
	DDRD |= (1<<DDRD5) ;
	DDRD |= (1<<DDRD6) ;
	DDRD |= (1<<DDRD7) ;	
	DDRD &= ~(1<<DDRD3) ;
	
	
	USART0_Init(MYUBRR);
	ADC_Init() ;
	TIMER0_Init() ;
	
	PIND = 0x00 ;
	
	sei() ;
	
	USART0_PutString("This is a Test STRING\n") ;
	
    /* Replace with your application code */
    while (1) 
    {
			
			if ( SW4_PRESSED )
				{
					LED11_ON ;
					LED12_ON ;
				}
			else
				{
					LED11_OFF ;
					LED12_OFF ;
				}
				
			if ( SW5_PRESSED)
				{
					LED13_ON ;
					LED14_ON ;
				}
			else
				{
					LED13_OFF ;
					LED14_OFF ;
				}			
				
			if (UART_Counter >= 200)
			{
				UART_Counter = 0 ;
				
					sprintf(My_Buffer,"ADC = %4d\n",ADC_Value) ;
					USART0_PutString(My_Buffer);
			}

    }
}

